# day06 集合(Collection, 迭代器, 增强for, 泛型, List子集)


## 集合的体系结构

* 集合: 
  * 是一个统称, 包含很多集合的实现类. 根据不同数据结构, 提供了不同的集合类
* 数据结构: 
  * 数据的组织和存储方式
* 体系结构: 
  * 将不同的集合实现类的共性向上抽取出父类, 从而形成一种体系
* 如何学习一种体系结构:
  * 从顶层开始学习共性
  * 从底层使用具体实现






## Collection接口中常用的功能

* `Collection`接口
	* 是单列集合体系的顶层
	* 如何创建Collection对象? 
		* 本身是接口, 实际使用的是子类对象. 使用多态, `Collection c = new ArrayList<>();`
	* 常用方法
		* 增
			* `boolean add(E e)`: 向集合中添加元素, 添加成功返回true, 添加失败返回false. 永远能够添加成功
		* 删
			* `boolean remove(Object o)`: 删除元素中的指定元素, 删除成功返回true, 删除失败返回false
			* `void clear()`: 清空集合中的元素
		* 判断
			* `boolean contains(Object o)`: 判断集合中是否包含指定元素, 包含返回true, 否则false
			* `boolean isEmpty()`: 判断集合是否为空集合, 即没有元素. 没有元素返回true, 否则false
		* 获取个数
			* `int size()`: 获取集合中元素的个数
		* 转换
			* `Object[] toArray()`: 将集合转换为Object[]
	## 迭代器的概述和测试


* `Iterator`接口: 
	* 迭代器
	* 作用: 
		* 提供遍历集合的安全方式
			获取迭代器:	
		* 使用集合对象调用`Interator<E> iterator()`方法
	* 成员方法
		* `boolean hasNext()`: 是否有下一个元素
		* `E next()`: 返回下一个元素
		* `void remove()`: 从迭代器指向的 collection 中移除迭代器返回的最后一个元素
## 并发修改异常

* 并发修改异常: 
	* ConcurrentModificationException
	* 并发: 
		* 并行发生, 即同时发生
	* 修改: 
		* 指的是会改变集合长度的操作
			* 增加元素
			* 删除元素
			* 清空元素
			* list的`set(int index, Object o)`方法可以修改集合中某个索引处的元素值, 但并不会改变集合的长度, 所以不会发生并发修改异常
	* 发生原因:
		* 迭代器依赖于集合, 相当于集合的一个副本, 当迭代器在操作时, 如果发现迭代器的副本和集合不一样, 则抛出并发修改异常
	* 如何避免:
		* 方式1: 不使用迭代器
		* 方式2: 在使用迭代器遍历时, 使用迭代器的方法修改
			* 添加元素: 
				* List接口中的`ListIterator listIterator()`获取用于List的迭代器, 然后调用ListIterator的add()方法
			* 删除元素: remove()
* List接口
	* 列表, 是Collection的子接口
	* 特点:
		* 元素存取有序, 元素可以重复, 有索引
### 使用LinkedList实现栈和队列

* 栈: 先进后出
	* 进: 元素添加到末尾, `addLast(E e)`
	* 出: 删除末尾的元素, `removeLast(E e)`
* 队列: 先进先出
	* 进: 元素添加到末尾, `addLast(E e)`
	* 出: 删除开头的元素, `E removeFirst()`

```java
/*
使用LinkedList模拟栈

分析: 
	栈有什么特点: 先进后出
		* 进: 元素添加到末尾, `addLast(E e)`
		* 出: 删除末尾的元素, `E removeLast()`
 */
public class Stack<E> {

	// 栈中需要保存数据, 使用LinkedList作为容器
	private LinkedList<E> linkedList;
	
	// 构造方法: 初始化成员
	public Stack() {
		linkedList = new LinkedList<>();
	}
	
	// 添加元素: 栈添加到末尾
	public void add(E e) {
		linkedList.add(e);
	}
	
	// 删除元素
	public E remove() {
		return linkedList.removeLast();
	}

	@Override
	public String toString() {
		return linkedList.toString();
	}
}

/*
使用LinkedList实现队列

分析:
	队列特点: 先进先出
		* 进: 元素添加到末尾, `addLast(E e)`
		* 出: 删除开头的元素, `E removeFirst()`
 */
public class Queue<E> {

	// 保存数据的成员
	private LinkedList<E> linkedList;
	
	// 构造方法: 初始化成员
	public Queue() {
		linkedList = new LinkedList<>();
	}
	
	// 添加
	public void push(E e) {
		linkedList.addLast(e);
	}
	
	// 删除
	public E pop() {
		return linkedList.removeFirst();
	}

	@Override
	public String toString() {
		return linkedList.toString();
	}
}

/*
 * 测试自己编写的栈和队列
 */
public class Test {

	public static void main(String[] args) {
		testStack();
		
		testQueue();
	}

	private static void testQueue() {
		Queue<Integer> queue = new Queue<>();
		
		queue.push(1);
		queue.push(2);
		queue.push(3);
		System.out.println(queue);
		
		int result1 = queue.pop();
		System.out.println("删除了" + result1 + ", 队列中有:" + queue);
		int result2 = queue.pop();
		System.out.println("删除了" + result2 + ", 队列中有:" + queue);
	}

	private static void testStack() {
		Stack<String> stack = new Stack<>();
		
		stack.add("hello");
		stack.add("world");
		stack.add("java");
		System.out.println(stack);
		
		String result1 = stack.remove();
		System.out.println("删除了" + result1 + ", 栈中有:" + stack);
		String result2 = stack.remove();
		System.out.println("删除了" + result2 + ", 栈中有:" + stack);
	}
}
```



## 今日总结

* 集合
	* 包含了不同的实现类, 向上抽取出了很多共性的接口, 形成了一个体系结构
* 数据结构
	* 数据的组织和存储方式
* 迭代器
	* 作用: 遍历集合
	* 并发修改异常:
		* 原因: 迭代集合时改变了集合的长度
		* 解决:
			* 不使用迭代器
			* 使用迭代器对象中的修改方法
* 泛型
	* 作用: 
		* 约束集合中元素的数据类型
		* 将运行时发生的类型转换异常提前到了编译时期
* 遍历集合的3种方式
	* 普通for循环
	* 迭代器
	* 增强for循环
		* 优点: 快速简便的遍历集合元素
		* 缺点: 不能在遍历过程中修改集合, 也没有索引可以使用
* 常见的数据结构
	* 数组
		* 特点:
			* 长度一旦确定则不可改变
			* 有整数索引
			* 只能存储同一个数据类型的元素
			* 既能存储基本数据类型, 又能存储引用数据类型
			* 增删慢, 查询快
	* 链表
		* 特点:
			* 增删快, 查询慢
	* 栈
		* 特点: 先进后出, FILO
	* 队列
		* 特点: 先进先出, FIFO

```java
// 增强for循环格式
for (元素的数据类型 变量名 : 数组或集合对象) {
	// 遍历出来的变量可以直接使用
}
```



```



Collection接口(单列集合体系的顶层)
	|	* boolean add(E e): 添加元素, 添加成功返回true, 否则false
	|	* void clear(): 清空集合中的元素
	|	* boolean contains(Object o): 判断集合中是否包含指定的对象
	|	* boolean isEmpty(): 判断集合中是否没有元素. 与null区分
	|	* boolean remove(Object o): 从集合中删除一个元素, 删除成功返回true, 否则false
	|	* int size(): 获取集合的长度(元素的数量)
	|	* Object[] toArray(): 将集合转换为Object[]数组
	|	* Iterator<E> iterator(): 获取集合的迭代器对象
    |- List接口(元素存取有序, 可重复, 有索引)  
		|	* void add(int index, E e): 添加元素到指定索引上
		|	* E remove(int index): 删除索引上的元素, 并返回
		|	* E set(int index, E e): 修改指定索引上的元素, 并返回被替换的元素
		|	* E get(int index): 获取指定索引上的元素
		|	* ListIterator listIterator(): 获取List特有的迭代器对象
	    |- ArrayList类(底层是数组, 查询快, 增删慢)
	    |- LinkedList类(底层是链表, 查询慢, 增删快)
				* void addFirst(E e): 添加元素到集合的开头
				* void addLast(E e): 添加元素到集合的末尾
				* E getFirst(): 获取集合开头的元素
				* E getLast(): 获取集合末尾的元素
				* E removeFirst(): 删除开头的元素, 并返回
				* E removeLast(): 删除末尾的元素, 并返回

Iterator接口
	|	* boolean hasNext(): 判断是否有下一个元素
	|	* E next(): 获取下一个元素
	|	* void remove(): 删除迭代器返回的最后一个元素
    |- ListIterator接口
			* void add(E e): 使用迭代器添加元素到集合
			* void remove(): 使用迭代器删除元素
