# day03 面向对象(接口, 多态)


## 接口的概述

* 接口 
	* 接口是完全抽象的类, 接口中所有方法都是抽象方法
* 类和接口的关系: 
	* 类实现接口
	* 实现接口的类称为`xx接口的实现类`
* 接口的作用: 
	* 为实现类扩展出接口具有的功能
	* 弥补Java只能单继承的局限性, 类只能被单继承, 而接口可以被多实现
* 定义接口:
	* 使用`interface`关键字替代`class`
	* 格式: `权限修饰符 interface 接口名 {}`
* 一个类如何实现接口:
	* `implements`关键字
	* 格式: `权限修饰符 class 类名 implements 接口名1, 接口名2... {}`
	* ## 接口的成员特点

* 接口的成员变量
  * 只能有自定义常量
  * 默认是`public static final`修饰的, 即使你不写或少写某个修饰符
* 接口的构造方法
  * 不能定义构造方法, 也不能创建对象
* 接口的成员方法
  * 必须都是抽象方法
  * 不能是static的
  * 默认是`public abstract`修饰, 即使你不写或少些某个修饰符
  	* 如果写也只能使用`public abstract`修饰, 因为接口必须通过实现类来实现抽象方法, 如果方法权限不足以调用, 则无法达成目的

## 接口和类的关系

* 类与类的关系: 
	* 继承关系: 子类继承父类
		* 单继承, 多层继承
* 类与接口的关系: 
	* 实现关系: 类实现接口
		* 多实现. (不能多层实现, 因为接口不能实现接口, 只能继承接口)
* 接口与接口的关系:
	* 继承关系: 子接口继承父接口
		* 多继承, 多层继承
* 实现或继承多个接口时, 接口名用`,`逗号分隔, 如
	* `interface C extends A, B {}`
	* `class Z implements A, B {}`
## 多态的概述和实现

* 多态: 
  * 一个事物可以具有多种表现形态
  * 是面向对象三大特征之一
  	* 面向对象三大特征:
  		* 封装
  		* 继承
  		* 多态
* 举例
  * 猫
    * 班里进来一只猫, 猫可以抓老鼠
    * 猫也是动物, 动物会睡觉, 猫继承了动物的睡觉行为
    * 所以, 猫作为猫类的对象去抓老鼠, 猫也可以作为动物类的对象去睡觉
  * 德鲁伊
    * 德鲁伊具有多种形态: 人型, 鸟类形态, 熊形态, 鱼形态
    * 无论哪种形态, 都可以吃饭, 睡觉
    * 还可以转变为特定形态使用具体技能
        * 德鲁伊可以转变为人型双腿跑步
        * 德鲁伊可以转变为鸟类形态飞行
        * 德鲁伊可以转变为熊形态吼叫
        * 德鲁伊可以转变为鱼形态潜水 
* 多态的前提
  * 子父类存在继承关系
  * 子类重写了父类的方法
  * 父类引用指向子类对象: `Animal cat = new Cat();`
* 动态绑定:
  * 对象调用的非静态的成员方法时, 是根据程序运行时该对象的实际类型确定的

```java  
## 多态的成员特点

* 类型以哪边为准? `Dad x = new Kid();`
	* 成员变量: 编译看左边, 运行看左边. `student.age`调用父类成员属性
	* 静态成员方法: 编译看左边, 运行看左边. `student.staticFunc();`调用父类静态方法
	* 非静态成员方法: 编译看左边, 运行看右边. `student.eat();`调用子类
* 简化记忆规律: **除了非静态成员方法运行时看右边, 其他无论编译还是运行都看左边**


```

## 多态的优缺点

* 优点
	* 提高代码复用性
	* 提高可维护性
	* 提高可扩展性
* 缺点
	* 父类无法直接访问子类特有成员

* 多态的应用:
	* 向上转型: 作为参数类型, 简化代码
	* 向下转型: 调用子类对象特有的方法
    ## 今日总结

* 接口
  * 概念: 接口是完全抽象的类, 接口中所有方法都是抽象方法
  * 作用:
  	* 扩展功能
  	* 弥补Java单继承的局限性, 因为接口可以多实现
  * 使用:
  	* 定义接口: `权限修饰符 interface 接口名 {}`
  	* 实现接口: `权限修饰符 class 实现类类名 implements 接口名1, 接口名2, ... {}`
  * 成员的特点:
  	* 成员变量
  		* 只能是静态常量, 默认使用`public static final`修饰, 即使不写或少写
  	* 构造方法
  		* 不能有构造方法, 也不能创建对象
  	* 成员方法
  		* 全都是抽象方法, 默认使用`public abstract`修饰, 即使不写或少写
  		* 不能是static的
  * 总结关系:
  	* 类和类: 继承关系, 单继承, 多层继承, 不能多继承
  	* 类和接口: 实现关系, 多实现
  	* 接口和接口: 继承关系, 多继承, 多层继承
  * 优点:
  	* 通过多实现打破单继承的局限性
  	* 接口可以提供规则
  	* 降低程序的耦合性
  * 接口和抽象类的异同
  	* 与类的关系不同
  		* 接口: 多实现关系
  		* 抽象类: 单继承, 多层继承
  	* 成员变量:
  		* 接口: 只能有静态常量, `public static final`
  		* 抽象类中: 可以成员变量, 也可以有常量
  	* 构造方法:
  		* 接口: 不可以有构造方法
  		* 抽象类: 可以有构造方法, 但不能创建对象, 用于初始化成员变量
  	* 成员方法:
  		* 接口: 只能是抽象方法, public abstract
  		* 抽象类: 可以有抽象方法, 也可以有非抽象方法, 也可以没有抽象方法
* 多态
  * 概念: 一个事物有多种形态
  * 前提: 
    * 子父类存在继承关系
    * 子类重写父类方法
    * 父类引用指向子类对象
  * 多态的成员特点:
    * 一句话: 除了成员方法在运行时看右边, 其他无论编译还是运行全看左边
    * 成员变量: 编译看左边, 运行看左边
    * 静态方法: 编译看左边, 运行看左边
    * 成员方法: 编译看左边, 运行看右边
  * 向上转型
    * 子类型转换为父类型(自动转换)
  * 向下转型
    * 父类型转换为子类型(强制转换)
  * 转型时的注意事项:
    * 只有子父类存在继承关系时, 才能自动向上转型
    * 只有子类对象本身就是要被强转的目标类型时, 才能够强转. 否则会抛出类型转换异常
    * 可以使用`instanceof`语句来判断对象的实际类型
      * 格式: `对象的变量名 instanceof 类名`
      * 解释: 如果是该类型则返回true, 如果不是则返回false
