
# day05 API(Object, System, 日期, 包装类, 正则)


## Object的toString()方法

* `java.lang.Object`类
	* 是类层次结构的根类(上帝)
	* 所有类都**直接或间接**继承了`Object`类
		* 举例:
			* `class Student {}`: Student默认继承了Object类
			* `class Student extends Person {}`: Student继承Person类, Person类继承Object类 
* 成员方法
	* `String toString()`: 返回对象的字符串表示方式, 如`com.itheima.Student@1200d083` 
		* 实现方式: `return getClass().getName() + "@" + Integer.toHexString(hashCode());`
		* 代码解释:
			* `getClass()`: 获取`Class`对象, 即字节码对象
			* `getName()`: 获取类的全名(包名.类名)
			* `Integer.toHexString()`: 将int整型数字转换为十六进制字符串
			* `hashCode()`: 获取hash值(通过内部地址转换成一个整数再转换为十六进制数) 
* 重写toString()方法
	* 作用: 任何类可以重写toString()方法来自定义输出内容. 开发调试时经常用, 主要是查看对象的属性值
	* Eclipse快捷键: `Alt + Shift + s` > `Generate toString()...` 
## Object类的equals方法

* Object类
	* `boolean equals(Object obj)`: 比较两个对象是否"相等"
* String类和Object类的`boolean equals(Object obj)`区别
	* String类的equals: 比较字符串内容是否相同. 重写了Object类的equals方法
	* Object类的equals: 使用`==`来比较, 基本数据类型比较值, 引用数据类型比较对象的地址值
* 快捷键:
	* 快速进入类的快捷键: 
		* `Ctrl + Shift + t`, 输入类名
		*  如果显示`Source not found`, 是找不到源码, 因为使用的是JRE环境没有源码
			*  点击`Attach Source` > 选择`External File` > 选择JDK安装目录下的`src.zip`
	* 查看一个类的方法树形图快捷键:
		* `Ctrl + o`
		* 输入方法名称后回车可以快速跳转到方法的定义位置
	* 生成equals方法: `Alt + Shift + s` > `Generate hashCode() and equals()...`
## System类概述

* `java.lang`包下的类, 不用导包就可以用
* `java.lang.System`类: 包含一些有用的类字段和方法, 不能被实例化
	* 类字段: 静态成员
		* `err`: 标准错误输出流
		* `in`: 标准输入流
		* `out`: 标准输出流
	* 常用静态方法
		* `static void arrayCopy(Object src, int srcPos, Object dest, int destPos, int length)`: 复制源数组中指定长度个元素到一个新数组中
			* `Object src`: 源数组 (被复制的数组)
			* `int srcPos`: 源数组索引 (从源数组的哪个索引开始复制)
			* `Object dest`: 目标数组 (要复制到哪个数组)
			* `int destPos`: 目标数组索引 (指定目标数组接收元素的索引位置)
			* `int length`: 长度 (要复制的元素个数)
		* `static long currentTimeMillis()`: 返回当前系统时间的毫秒值, 从零时区1970-01-01 00:00:00开始算
		* `static void exit(int status)`: 终止虚拟机的运行, 0表示正常退出, 非0表示异常退出


## System类的静态方法: arrayCopy()

* `static void arrayCopy(Object src, int srcPos, Object dest, int destPos, int length)`: 复制源数组中指定长度个元素到一个新数组中
	* src: 源数组
	* srcPos: 指定从源数组的哪个索引开始复制
	* dest: 目标数组
	* destPos: 指定目标数组接收元素的索引位置 
	* length: 目标数组接收元素的个数
* 注意: 复制时注意数组长度, 不要超出索引, 否则会出现数组越界异常


## System类的其他方法

* `static long currentTimeMillis()`: 以毫秒值返回当前系统时间, 毫秒值从 0时区的1970-01-01 00:00:00 开始计算  
	* 示例:
		* `1970-01-01 00:00:00.000` = 0
		* `1970-01-01 00:00:00.500` = 500
		* `1970-01-01 00:00:01.000` = 1000
	* 这种时间表示方法也叫作`时间戳`, 按现在的时间计算分为10位和13位的, 10位精确到秒, 13位精确到毫秒
		* 10位: 1494992791    = `2017/5/17 11:46:31`
		* 13位: 1494992791000 = `2017/5/17 11:46:31`
* `static void exit(int status)`: 终止虚拟机的运行
	* status参数取值惯例: 非0表示异常终止, 0表示正常终止
	* ## Date类的概述和构造

* **注意**: 
	* 不要导错包! 有2个包中都有Date类, 一个是`java.sql.Date`, 另一个是`java.util.Date`, **我们用的是`java.util.Date`**
* `java.util.Date`类: 日期类, 表示特定的瞬间, 可以通过方法来设定所表示的时间, 可以表示任意的时间
	* 构造方法
		* `Date Date()`: 创建Date对象, 并表示当前系统时间
		* `Date Date(long date)`: 创建Date对象, 使用指定的时间
	* 成员方法
		* `String toLocalString()`: 转换为本地时间格式的字符串
* `已过时`是什么意思? 
	* 可能有缺陷, 不推荐使用, 有新的替代方法. 但仍然可以用. 调用已过时的方法, 方法上会有一个删除线


## Date类的常用方法

* 成员方法
	* `void setTime(long time)`: 设置Date对象的时间 
	* `long getTime()`: 获取Date对象中保存的时间毫秒值
		* 注意, 无论何时调用, 都是获取Date对象中已经存储的那一瞬间的时间, 而不是调用方法时的时间


## SimpleDateFormat类概述

* 思考如何通过API文档查找想要的信息
* `java.text.SimpleDateFormat`类: 专门用于格式化和解析时间
	* 构造方法:
		* `SimpleDateFormat SimpleDateFormat(String pattern)`: 创建对象同时指定格式
	* 成员方法:
		* 格式化: Date对象 -> 指定格式的字符串, 如`2017-05-01 12:34:12`
			* `String format(Date date)`: 格式化Date对象返回字符串
		* 解析: 字符串 -> Date对象
			* `Date parse(String source)`: 将字符串解析为Date对象
				* 该方法抛出`ParseException`解析异常, 因为字符串可能不匹配指定的格式

```
// SimpleDateFormat类作为中介, 可以将Date和String相互转换

Date对象 <- SimpleDateFormat对象 -> String对象
```


## SimpleDateFormat功能测试

* `java.text.SimpleDateFormat`类: 专门用于格式化和解析时间
    * 构造方法:   
        * `SimpleDateFormat SimpleDateFormat(String pattern)`: 创建对象同时指定格式
    * 成员方法
        * `String format(Date date)`: 将Date对象格式化为字符串
        * `Date parse(String source)`: 将字符串解析为Date对象
    * 常用日期模式: 详细可见API文档
      * `y`: 年
      * `M`: 月
      * `m`: 分
      * `D`: 年中的天
      * `d`: 月中的天
      * `H`: 小时(24小时制)
      * `K`: 小时(12小时制)
      * `S`: 毫秒
      * `s`: 秒
      * `E`: 星期
      * 示例: `yyyy-MM-dd E HH:mm:ss.SSS`, 结果为`2016-04-01 星期五 17:29:15.868`
    * 注意:
        * 每个`SimpleDateFormat`对象都有一个对应的模式, 无论`format()`还是`parse()`, 都是用该对象的模式操作
        * 如: `new SimpleDateFormat("yyyy-MM-dd")`和`new SimpleDateFormat("yyyy年MM月dd日")`两个对象的模式就不一样

* 5分钟练习: 日期格式化和解析
	* 继续使用项目s2-day05, 建包com.itheima.practice_04
	* 建类Test
		* 练习Date转字符串
			* 使用空参构造创建Date对象
			* 创建SimpleDateFormat对象, 格式为: yyyy年MM月dd日
			* 调用format方法将date对象格式化, 打印格式化后的日期字符串
		* 练习字符串转Date
			* 定义字符串
			* 创建新的SimpleDateFormat对象, 格式为: yyyy-MM-dd
			* 调用parse方法将字符串解析为Date对象
			* 打印date对象调用getTime()方法获得的时间

```java
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

/*
按要求实现以下需求
将当前日期格式化为”xxxx年xx月xx日”并打印
已知日期字符串”2000-10-01”
将该日期字符串格式化为日期对象,获取该日期对象的毫秒值并打印
 */
public class Test {

	public static void main(String[] args) throws ParseException {
		// 创建Date对象表示当前日期
		Date now = new Date();
		// 创建SimpleDateFormat对象, 并指定模式 xxxx年xx月xx日
		SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy年MM月dd日");
		// 调用format()方法将Date格式化为String
		String result = sdf1.format(now);
		System.out.println(result);   // 2018年05月24日
		
		// 已知字符串
		String s = "2000-10-01";
		// 创建SimpleDateFormat对象, 并指定模式: 2000-10-01
		SimpleDateFormat sdf2 = new SimpleDateFormat("yyyy-MM-dd");
		// 调用parse()方法将String解析为Date对象
		Date date = sdf2.parse(s);
		// 获取毫秒值
		long millis = date.getTime();
		System.out.println(millis);  
	}
}
```





---=







## 求出你来这个世界上多少天

* 分析
	* 我们常见的生日和当前日期, 通常都用字符串表示, 如`2000-01-01`, `2017-04-01`
	* 2个字符串日期无法直接相减计算, 怎么能计算出相差多少天呢?
	* 能表示时间的有`日期字符串`, `Date对象`, `long类型的毫秒值`, 这3者只有毫秒值可以相减, 所以能否将日期字符串转换为long毫秒值?
	* 转换思路
		* 字符串到毫秒: `String -> SimpleDateFormat -> Date -> long` 
		* 毫秒到天: `天 = 毫秒 / 1000 / 60 / 60 / 24`
* 5分钟练习: 求出你来这个世界上多少天
	* 继续使用项目s2-day05, 建包com.itheima.practice_05
	* 建类Test
		* 使用自己的生日定义字符串
		* 创建SimpleDateFormat对象, 格式为: yyyy-MM-dd
		* 解析生日字符串为Date对象
		* 使用空参创建新的Date对象
		* 当前Date对象调用getTime方法减去生日Date调用getTime方法获取时间差
		* 将时间差毫秒值换算为天数: time / 1000 / 60 / 60 / 24
		* 打印出来
	* 扩展: 
		* 假设你65岁退休, 算算从现在开始还能挣几个月的钱?
		* 如果每月15K工资, 总共能挣多少?
		* 注意有坑, int类型超出界限变成了负数

```java
/*
练习求出你来这个世界上多少天的案例
 */
public class Test {

	public static void main(String[] args) throws ParseException {
		// 出生日期
		String birthdayStr = "2018-05-23";
		// 今天
		String nowStr = "2018-05-24";
		// 创建SimpleDateFormat
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
		// 将两个日期转换为Date
		Date birthdayDate = sdf.parse(birthdayStr);
		Date nowDate = sdf.parse(nowStr);
		// 转换为毫秒值
		long birthdayMillis = birthdayDate.getTime();
		long nowMillis = nowDate.getTime();
		// 计算时间差
		System.out.println((nowMillis - birthdayMillis) / 1000 / 60 / 60 / 24);
	}
}
```


## Calendar的概述和测试

* `java.util.Calendar`类: 日历类, 也是处理时间和日期的类, 用于替代Date类
  * 创建对象
    * 静态方法: `static Calendar getInstance()`
    	* 示例: `Calendar c = Calendar.getInstance();`
  * 静态字段: 表示时间的某个部分
    * `static YEAR`: 年份
    * `static MONTH`: 月份. 注意月份数值是0-11
    * `static DAY_OF_MONTH`: 日期
    * `static HOUR`: 小时(12小时制)
    * `static HOUR_OF_DAY`: 小时(24小时制)
    * `static MINITE`: 分钟
    * `static SECOND`: 秒
  * 成员方法
    * `int get(int field)`: 返回给定日历字段的值
      * 例如:
        * 获取日期: `int day = calendar.get(Calendar.DAY_OF_MONTH);`
    * `void set(int field, int value)`: 将给定的日历字段设置为给定值
      * 例如:
        * 设置年: `calendar.set(Calendar.YEAR, 2000);`
        * 设置月: `calendar.set(Calendar.MONTH, 0);`
    * `void add(int field, int amount)`: 为给定的日历字段添加或减去指定的时间量, 如时间字段是天, 增加n天, 时间字段是秒, 增加n秒. 减少用负数
      * 例如:
        * 当前日期增加一天: `calendar.add(Calendar.DAY_OF_MONTHY, 1);`
        * 当前日期减少一天: `calendar.add(Calendar.DAY_OF_MONTHY, -1);`

* 5分钟练习:
  * 继续使用项目s2-day05, 建包com.itheima.practice_06
  * 建类Test
  	* 使用静态方法获取Calendar对象
  	* 测试get方法获取年月日并打印
  	* 测试add方法修改日期并打印
  	* 测试set方法修改日期并打印

```java
/*
练习以下方法
int get(int field) 获取当前年月日打印
void add(int field, int amount) 修改日期后打印
void set(int field, int value)设置日期后打印
 */
public class Test {

	public static void main(String[] args) {
		// 获取Calendar对象
		Calendar calendar = Calendar.getInstance();
		
		// int get(int field) 获取当前年月日打印
		int year = calendar.get(Calendar.YEAR);
		int month = calendar.get(Calendar.MONTH) + 1;  // 月份从0开始
		int day = calendar.get(Calendar.DAY_OF_MONTH);
		System.out.println(year + "年" + month + "月" + day + "日");
		
		// void add(int field, int amount) 修改日期后打印
		calendar.add(Calendar.DAY_OF_MONTH, -1);  // 向前调整一天
		year = calendar.get(Calendar.YEAR);
		month = calendar.get(Calendar.MONTH) + 1;  // 月份从0开始
		day = calendar.get(Calendar.DAY_OF_MONTH);
		System.out.println(year + "年" + month + "月" + day + "日");
		
		// void set(int field, int value)设置日期后打印
		calendar.set(Calendar.DAY_OF_MONTH, 1);  // 设置日期为1号
		year = calendar.get(Calendar.YEAR);
		month = calendar.get(Calendar.MONTH) + 1;  // 月份从0开始
		day = calendar.get(Calendar.DAY_OF_MONTH);
		System.out.println(year + "年" + month + "月" + day + "日");
	}
}
```





---







## 包装类的概述和基本使用

* 为什么要学习包装类? 
  * 包装类是封装了基本数据类型的类, 提供了更多方法和变量, 便于进行数据的判断, 计算, 转换等操作
  * 同时将基本类型的使用转换为了类的面向对象方式
* 基本数据类型与其包装类的对应关系
  * byte: Byte
  * short: Short
  * char: **Character**
  * int: **Integer**
  * long: Long
  * float: Float
  * double: Double
  * boolean: Boolean
* `java.lang.Integer`
	* 构造方法：
		* `Integer(int value)`: int转Integer
		* `Integer(String s)`: String转Integer
	* String和int的常用转换方法
		* String转int
			* 方式1：`int intValue()`
			* 方式2： `static int parseInt(String s)`
		* int转String
			* 方式1： `+ ""`
			* 方式2：`String toString()`


* 扩展: 包装类有什么用?
  * 能将基本类型的数据通过自动装箱存入集合
  * 常用一些转换方法

  * 包装类默认值是`null`, 数字基本数据类型默认值是`0`, 而在实际开发中0也有可能是正常数值, 比如温度, 人数等. 有时我们需要判断一个成员变量否进行过初始化, 0可能会被认为是已经初始化的正常数据, 所以用包装类是否为null判断更合适

* 5分钟练习:
	* 继续使用项目s2-day05, 建包com.itheima.practice_07
	* 建类Test
		* 测试Integer类的方法


```java
/*
测试Integer类的方法
 Integer(int  value)
 Integer(String s)
 int intValue()
 static int parseInt(String s)
 static String toString(int i)
 */
public class Test {

	public static void main(String[] args) {
		// Integer(int  value): int转Integer
		Integer i1 = new Integer(1);
		System.out.println(i1);
		
		// Integer(String s): String转Integer
		Integer i2 = new Integer("2");
		System.out.println(i2);
		
		// int intValue(): Integer转int
		int i3 = i1.intValue();
		System.out.println(i3);
		
		// static int parseInt(String s): String转int
		int i4 = Integer.parseInt("4");
		System.out.println(i4);
		
		// static String toString(int i): int转String
		String s5 = Integer.toString(5);
		System.out.println(s5);
	}
}
```


## 包装类的自动装箱和自动拆箱

* 箱=包装
* 什么是自动装箱? 
	* 概念: 基本数据类型可以自动转换为包装类
	* 示例: `Integer i = 10;`
	* 原理: 相当于执行`Integer i = new Integer(10);`
		* 或`Integer i = Integer.valueOf(10);`
* 什么是自动拆箱? 
	* 概念: 包装类可以自动转换为基本数据类型
	* 示例: `Integer i = 10; int a = i;`
	* 原理: 相当于执行`int a = i.intValue();`
* 自动拆装箱的应用场景
	* 集合只能存储引用数据类型, 但有了自动拆装箱, 基本类型也可以转换为包装类存入集合中
		* `list.add(1)`: 1自动装箱
		* `int i = list.get(0);`: i自动拆箱



---





## 正则表达式概述

* 什么是正则表达式? 
  * 是一种用于匹配字符串的规则
  * 可用于判断字符串是否符合某种规则, 比如是否是邮箱, 是否是手机号等等
* `java.util.regex.Pattern`类: 
  * 创建正则表达式模板, 内有正则表达式规则
* 常用规则: 具体查看API文档的Pattern类
  * 字符:
    * `x`: 一个字符x      String s ="a"    "a"
    * `\\`: 表示`\`反斜杠. 
      * 为什么有2个? 第一个`\`是转义的作用
    * `[abc]`: a或b或c, 即中括号字母中的其中一个  [abc]{2}  "aa"
    * `[^abc]`: 除a, b, c之外的任何字符. 
      * `^`表示否定
    * `[a-zA-Z]`: 小写英文a到z, 以及大写英文A到Z. 
      * `-`表示范围
  * 预定义字符
    * `.`: 任何字符(对于行结束符, 可能匹配, 也可能不匹配)
    * `\d`: 数字0-9. 相当于`[0-9]`
    	* d是digital的缩写
    * `\D`: 非数字. 相当于`[^0-9]`
    * `\s`: 空白字符. 相当于`[ \t\n\x0b\f\r]`, 包括空格(第一个空就是空格)
    	* s是space的缩写
    * `\S`: 非空白字符. 相当于`[^\s]`
    * `\w`: 单词字符. 相当于`[a-zA-Z_0-9]`
    	* w是word的缩写
    * `\W`: 非单词字符. 相当于`[^\w]`
  * Greedy数量词
    * `X?`: X这个字符出现一次或零次  
    * `X*`: X这个字符出现零次或多次, 包括一次
    * `X+`: X这个字符出现一次或多次
    * `X{n}`: X这个字符恰好出现n次
    * `X{n, }`: X这个字符至少出现n次
    * `X{n, m}`: X这个字符至少出现n次, 但不超过m次
* 什么叫转义?
  * 就是转换含义. 因为正则中有一些特殊字符, 如`+`, `-`, `*`, `\`等都有特殊意义, 所以在作为普通字符使用时必须转义, 如:
  	* 用`\\`转义: 因为`\x`可能表示某种符号, 如`\t`所以要用2个反斜杠
  		* `+`表示一个或多个, `\\+`表示加号字符
  		* `-`表示范围, `\\-`表示减号字符
  		* `*`表示零个或多个, `\\*`表示星号字符
  	* 用`\`转义:
  		* `\`要结合某些字符进行匹配, `\\`表示斜杠字符


## 练习: 判断字符串中间的字符是否是元音字母

* String类
	* `boolean matches(String regex)`: 当前字符串是否匹配指定的正则表达式  reg ex   regular expression


* 5分钟练习: 判断字符串中是否在中间有元音字母
	* 继续使用项目s2-day05, 建包com.itheima.practice_08
	* 建类Test
		* 定义元音字母的正则表达式: `String pattern = "\\w+[aeiouAEIOU]+\\w+";`
		* 键盘录入字符串
		* 调用String类的`matches(String regex)`方法判断是否包含元音字母
		

```java
/*
练习判断字符串中是否包含元音
Qaq    “\\w[aeiouAEIOU]\\w”
 */
public class Test {

	public static void main(String[] args) {
		String s = "qbq";
		System.out.println(check(s));
	}
	
	public static boolean check(String s) {
		return s.matches("\\w[aeiouAEIOU]\\w");
	}
}
```


## 切割字符串

* String类
  * `String[] split(String regex)`: 按照正则表达式切分当前字符串为多个部分, 返回每个部分组成的字符串数组, 其中不包含分隔符

* 5分钟练习: 使用正则切割字符串
  * 继续使用项目s2-day05, 建包com.itheima.practice_09


```java
/*
练习利用正则表达式切割字符串
"aa,bb,cc"
"-1 99 4 23"
"-1        99 4   23“
 */
public class Test {

	public static void main(String[] args) {
		String[] arr1 = "aa,bb,cc".split(",");
		System.out.println(Arrays.toString(arr1));
		
		String[] arr2 = "-1 99 4 23".split(" ");
		System.out.println(Arrays.toString(arr2));
		
		String[] arr3 = "-1        99 4   23".split(" +");
		System.out.println(Arrays.toString(arr3));
	}
}
```



------------------

## 今日扩展

* 使用包装类比较数值时的注意事项
	* ==在比较引用数据类型时, 默认比较的是地址值
	* 包装类也是引用类型, 对象使用==比较, 比较的是地址值, 所以最终还是比较对象地址值是否相同
	* 而包装类在使用自动装箱时, 创建的包装类对象实际是这样执行的:
		* `Integer x1 = 128;`, 实际上是调用`包装类.valueOf()`方法, 相当于`Integer x1 = Integer.valueOf(128);`, 返回的是一个Integer对象
	* 所以包装类用==比较是否相等, 实际比较的是`包装类.valueOf()`方法返回的对象地址值是否相等
		* 除了Short, Integer和Long包装类, 其他包装类的`valueOf()`方法, 内部都是`new 包装类(值);`, 相当于创建了一个新的对象, 所以两个对象地址值肯定不同
		* 而Short, Integer和Long包装类的`valueOf()`方法, 内部提前对`-128~127`范围的数创建了Short, Integer或Long对象, 并存入了数组中进行了缓存, 所以每次调用`valueOf()`方法, 实际上是从缓存数组中拿出相同的包装类对象, 所以在`-128~127`范围的Short, Integer, Long包装类的两个对象地址值是相同的
	* 那超出-128~127如何比较值是否相等呢? 
		* 可以使用equals()方法比较, 包装类对equals()方法都进行了重写, 调用equals()方法时, 是使用自动拆箱获取基本类型的值进行比较的
* 总结: 
	* -128~127范围内, 所有数值的包装类使用==比较数值是否相等是可以的, 也可以使用equals()方法比较
	* 超出-128~127范围, 建议使用包装类的equals()方法比较
	* 为了省事, 都使用equals()方法比较, 这更为面向对象

```java
// 基本类型使用==比较值
int a1 = 128;
int b2 = 128;
System.out.println(a1 == b1); // true, 基本数据类型==比较值是否相等
int a2 = 127;
int b2 = 127;
System.out.println(a2 == b2); // true, 基本数据类型==比较值是否相等

// 包装类是引用类型
Integer x1 = 128;
Integer y1 = 128;
System.out.println(x1 == y2);  // false, 包装类==需要先自动拆箱(调用valueOf()), 然后再比较. 128超出缓存范围, 所以比较对象的地址值
Integer x2 = 127;
Integer y2 = 127;
System.out.println(x2 == y2);  // true, 包装类==需要先自动拆箱(调用valueOf()), 然后再比较. 127没有超出缓存范围, 所以取出缓存的对象比较地址值
```

```java
// Byte类中的valueOf()方法定义. 虽然Byte也进行了缓存, 但Byte本身范围也是-128~127, 所以对于值的比较没有特殊影响
public static Byte valueOf(byte b) {
    final int offset = 128;
    return ByteCache.cache[(int)b + offset];
}

// Character类中的valueOf()方法定义. 虽然Character也进行了缓存, 但Character本身范围也是-128~127, 所以对于值的比较没有特殊影响
public static Character valueOf(char c) {
    if (c <= 127) { // must cache
        return CharacterCache.cache[(int)c];
    }
    return new Character(c);
}

// Short类中的valueOf()方法定义. 对-128~127数值的对象进行了缓存
public static Short valueOf(short s) {
    final int offset = 128;
    int sAsInt = s;
	// 如果数值在-128到+127, 则从缓存数组中取出Integer对象
    if (sAsInt >= -128 && sAsInt <= 127) { // must cache
        return ShortCache.cache[sAsInt + offset];
    }
	// 如果不在这个范围内, 则new新的对象
    return new Short(s);
}

// Integer类中的valueOf()方法定义. 对-128~127数值的对象进行了缓存
public static Integer valueOf(int i) {
    assert IntegerCache.high >= 127;
	// 如果数值在-128到+127, 则从缓存数组中取出Integer对象
    if (i >= IntegerCache.low && i <= IntegerCache.high)
        return IntegerCache.cache[i + (-IntegerCache.low)];
	// 如果不在这个范围内, 则new新的对象
    return new Integer(i);
}

// Long类中的valueOf()方法定义. 对-128~127数值的对象进行了缓存
public static Long valueOf(long l) {
    final int offset = 128;
	// 如果数值在-128到+127, 则从缓存数组中取出Integer对象
    if (l >= -128 && l <= 127) { // will cache
        return LongCache.cache[(int)l + offset];
    }
	// 如果不在这个范围内, 则new新的对象
    return new Long(l);
}

// Float类中的valueOf()方法定义. 直接创建了新对象
public static Float valueOf(float f) {
    return new Float(f);
}

// Double类中的valueOf()方法定义. 直接创建了新对象
public static Double valueOf(double d) {
    return new Double(d);
}
```


* 同学们如果有兴趣可以封装以下工具类, 以后可以在工作中使用
	* 字符串校验工具类: 使用正则表达式实现, 常用于校验用户输入的信息
		* 邮箱地址
		* 身份证号
		* IP地址
		* 手机号
		* 邮政编码
	* 日期工具类
		* 计算天数: 常用于显示用户注册天数
			* 传入指定日期, 返回到现在一共经过了多少天
		* 计算年龄:
			* 传入指定日期, 返回到现在有多少岁
		* 判断某一年是否是闰年
			* 传入指定日期, 返回是否为闰年(能被4和100整除)
			* `return (year % 4 == 0 && year % 100 != 0);`
		* 以后可能涉及到ISO格式日期和UTC格式日期的转换, 以及不同时区日期的转换, 慢慢积累


## 今日总结

* Object
  * 是所有类的根类, 所有类默认都直接或间接的继承自它
  * 成员方法:
  	* `String toString()`: 返回对象的字符串表示方式
  		* 重写toString()的作用: 改变对象的字符串显示方式
  	* `boolean equals(Object o)`: 比较两个对象是否"相等"
  		* Object类中实现方式: `==`, 比较对象的地址值是否相同
  		* 重写equals()方法: 改变比较的规则. 可以直接生成
  * 获取字节码的3种方式
  	* `Class 对象.getClass()`
  	* `类名.class`
  	* `Class Class.forName("类的全名")`
  * 类的字节码对象注意点:
  	* 同一个类的字节码对象只有一个, 地址值也是相同的
  	* 区分Class和class, 前者是类, 后者是关键字
* System类
  * java.lang包下, 不用导包
  * 常用静态方法
  	* `static void arrayCopy(Object src, int srcPos, Object dest, int destPos, int length)`: 复制数组
  	* `static long currentTimeMillis()`: 返回当前系统时间的毫秒值, 从1970-01-01 00:00:00
  	* `static void exit(int status)`: 终止虚拟机的运行. 传入0是正常退出, 非0是异常退出
* Date类
  * 注意导包: java.util.Date
  * 表示特定的瞬间
  * 构造方法
  	* `Date Date()`: 创建Date对象, 并表示当前系统时间
  	* `Date Date(long timestamp)`: 创建Date对象, 使用指定的时间
  * 成员方法
  	* `String toLocalString()`: (已过时)转换为本地时间格式的字符串
  	* `void setTime(long time)`: 设置Date对象的时间
  	* `long getTime()`: 获取Date对象中保存的时间毫秒值.
* SimpleDateFormat类
  * 作用: 
  	* String转Date
  	* Date转String
  * 构造方法
  	* `SimpleDateFormat(String pattern)`: 创建SimpleDateFormat对象, 并设置指定的格式
  * 常用2个成员方法
  	* `String format(Date date)`: 格式化Date对象返回字符串
  	* `Date parse(String source)`: 将字符串解析为Date对象
  		* 字符串必须符合模式, 否则抛出异常
  * 常用日期模式
  	* `y`: 年
  	* `M`: 月
  	* `m`: 分
  	* `D`: 年中的天
  	* `d`: 月中的天
  	* `H`: 小时(24小时制)
  	* `K`: 小时(12小时制)
  	* `S`: 毫秒
  	* `s`: 秒
  	* `E`: 星期
* Calendar类
  * 替代Date类
  * 创建对象
  	* `static Calendar getInstance()`: 获取日历对象
  * 静态字段: 表示时间的某个部分
  	* `Calendar.YEAR`: 年份
  	* `Calendar.MONTH`: 月份. 注意月份数值是0-11
  	* `Calendar.DAY_OF_MONTH`: 日期
  	* `Calendar.HOUR`: 小时(12小时制)
  	* `Calendar.HOUR_OF_DAY`: 小时(24小时制)
  	* `Calendar.MINITE`: 分钟
  	* `Calendar.SECOND`: 秒
  * 成员方法
  	* `get(时间字段)`: 获取指定的时间部分
  	* `set(时间字段, 指定数值)`: 使用指定数值设置指定的时间部分
  	* `add(时间字段, 增加的时间值)`: 根据时间字段的单位增加指定的时间值, 如时间字段是天, 增加n天, 时间字段是秒, 增加n秒. 减少用负数
* 包装类
  * 作用:
    * 封装了基本数据类型的类, 提供了更多了方法和变量
    * 把基本数据类型的使用方式转换为面向对象的使用方式
  * 基本数据类型和引用数据类型的对应关系
    * byte: Byte
    * short: Short
    * char: Character
    * int: Integer
    * long: Long
    * float: Float
    * double: Double
    * boolean: Boolean
  * Integer类的转换方法
    * int转Integer: `Integer Integer(int value)`
    * String转Integer: `Integer Integer(String s)`
    * Integer转int: 
    	* `int intValue()`: 将Integer对象的值转换为int类型
    * String转int: 
    	* `static int parseInt(String s)`: 将字符串转换为int
    	* `static Integer valueOf(String s)`
    * int转String
    	* `"" + int`
    	* Integer对象的`toString()`方法
    	* `static String toString(int i)`: 将int转换为String
  * 包装类是引用类型, 所以默认值是`null`
  * 自动装箱
    * 基本数据类型可以自动转换为包装类
    * 举例: `Integer i = 10;` 相当于 `Integer i = new Integer(10);`或`Integer i = Integer.valueOf(10);`
  * 自动拆箱
    * 包装类可以自动转换为基本数据类型
    * 举例: `int a = i.intValue();`
  * 自动装箱拆箱的应用场景
    * 集合存储基本数据类型就是自动装箱, 本质上集合中存储的都是引用数据类型
* 正则表达式
  * 概念: 一种用于匹配字符串的规则
  * String类
  	* `boolean matches(String regex)`: 是否匹配指定的正则
  	* `String[] split(String regex)`: 按照正则表达式分割字符串
  * 转义:
  	* 转义使用`\`, 正则中有的字符需要转义(. + - *, 预定义字符要写两个斜杠)
  * 常用规则(查看API中的Pattern类)
  	* 字符:
  		* `x`: 一个字符x
  		* `\\`: 表示`\`反斜杠. 
  			* 为什么有2个? 第一个`\`是转义的作用
  		* `[abc]`: a或b或c, 即中括号字母中的其中一个
  		* `[^abc]`: 除a, b, c之外的任何字符. 
  			* `^`表示否定
  		* `[a-zA-Z]`: 小写英文a到z, 以及大写英文A到Z. 
  			* `-`表示范围
  	* 预定义字符
  		* `.`: 任何字符(对于行结束符, 可能匹配, 也可能不匹配)
  		* `\d`: 数字0-9. 相当于`[0-9]`
  			* d是digital的缩写
  		* `\D`: 非数字. 相当于`[^0-9]`
  		* `\s`: 空白字符. 相当于`[ \t\n\x0b\f\r]`, 包括空格(第一个空就是空格)
  			* s是space的缩写
  		* `\S`: 非空白字符. 相当于`[^\s]`
  		* `\w`: 单词字符. 相当于`[a-zA-Z_0-9]`
  			* w是word的缩写
  		* `\W`: 非单词字符. 相当于`[^\w]`
  	* Greedy数量词
  		* `X?`: X这个字符出现一次或零次
  		* `X*`: X这个字符出现零次或多次, 包括一次
  		* `X+`: X这个字符出现一次或多次
  		* `X{n}`: X这个字符恰好出现n次
  		* `X{n, }`: X这个字符至少出现n次
  		* `X{n, m}`: X这个字符至少出现n次, 但不超过m次
