## 顺序结构的基本使用

* 流程控制语句分类:
	1. 顺序结构
	2. 选择结构
	3. 循环结构


## 顺序结构

* 顺序结构是最基本的流程
* 顺序结构的特点: 从上往下依次进行


## 选择结构
### if语句
#### if语句第一种格式的基本使用

* 选择结构: 
	* 也被称为分支结构
* 选择结构有2种:
	1. if语句
	2. switch语句
	### switch语句的格式和执行流程

* switch语句的格式

```java
switch(表达式) {
	case 值1:
		语句体1;
		break;
	case 值2:
		语句体2;
		break;
	...
	case 值n:
		语句体n;
		break;
	default:
		语句体n+1;
		break;
}
```

* 说明
	* switch表达式允许的数据类型: 
		* byte, short, char, int
			* 都是可隐式转换为int的类型, 因为byte, short, char可以隐式转换为int, 所以也可以使用
		* JDK5起可以是枚举类型
		* JDK7起可以是String
		* 小括号中可以是常量, 也可以是变量
	* case后面的值: 
		* 用来和表达式的值进行匹配
		* 值只能是常量
		* 值的类型应该和switch小括号中值的类型一致
	* break: 中断
	* default: 当所有case都不匹配时, 执行default
* 执行流程
	* 首先计算表达式的值
	* 然后执行符合的case
	* 如果没有符合的case, 则执行default
## 循环控制语句

* 循环语句: 在满足循环条件的情况下, 反复执行某段代码.
	* 循环体: 这段被重复执行的代码
* 循环的停止: 
	* 在合适的时候将循环判断条件修改为false, 从而结束循环. 否则循环将一直执行下去, 形成死循环
* 循环语句的组成
	* 初始化语句: 一条语句(一条语句可以初始化多个变量), 这些语句完成一些初始化操作
	* 判断条件语句: boolean表达式, 这个表达式能决定是否执行循环体
	* 循环体语句: 要重复做的事情
	* 控制条件语句: 控制循环结束
* 3种循环结构语句
	1. for循环
	2. while循环
	3. do...while循环
4. * 注意: **初始化语句在循环中只执行一次**
## 控制循环语句

* 2个控制循环语句:
    * `break`: 直接结束循环, 或switch语句
    * `continue`: 结束本次循环, 继续执行下一次

### break跳转控制语句的概述和基本使用

* break的使用场景: 不能单独使用, 必须用于switch或循环中
* break的作用:
    1. 在switch语句中, 用于结束case
    2. 在循环中, 用于结束所在的循环


### continue跳转控制语句

* continue的使用场景: 循环中
* continue的作用: 结束本次循环, 继续下一次的循环
## 今日扩展

* switch语句的case穿透
	* 如果switch的case语句中没有break, 则会继续执行下一个case的代码, 这称为case穿透
	* case穿透有时会导致代码出现问题, 但有时我们也利用case穿透简化代码if语句不写大括号的格式
	* 当if语句不写大括号时, 只有if语句后的第一条语句算作if语句的语句体, 其余不算

```java
// 示例
if (false) {
	System.out.println("这条语句不会被执行");
	System.out.println("这条语句不会被执行");
} 

// 去掉大括号后
if (false) 
	System.out.println("这条语句不会被执行");
	System.out.println("这条语句会被执行");
## 今日总结

* 流程控制语句
	* 顺序结构
	* 选择结构: 用于根据不同条件执行不同的代码
		* if语句
			* 3种格式
			* else...if, else语句可以省略
		* switch语句
			* case穿透
			* switch表达式允许的数据类型: 
				* byte, short, char, int
					* 都是可隐式转换为int的类型, 因为byte, short, char可以隐式转换为int, 所以也可以使用
				* JDK5起可以是枚举类型
				* JDK7起可以是String
				* 小括号中可以是常量, 也可以是变量
			* case后面的值: 
				* 用来和表达式的值进行匹配
				* 值只能是常量
				* 值的类型应该和switch小括号中值的类型一致
			* break: 中断
			* default: 当所有case都不匹配时, 执行default
		* if语句适用于有范围的场景, 或者具体值的场景
		* switch语句适用于具体值场景, 不适用于范围的场景
	* 循环结构: 用于在满足循环条件的情况下, 重复执行某些代码
		* for循环
			* 适用于知道循环次数的场景
		* while循环
			* 适用于不知道循环次数的场景
			* 至少执行0次循环体
		* do...while循环
			* 适用于不知道循环次数的场景
			* 至少执行1次循环体
		* 循环控制语句: 控制循环的跳过或结束
			* break
				* 适用场景: switch语句, 循环
				* 作用: 结束switch语句或所在循环
			* continue
				* 适用场景: 循环
				* 作用: 结束本次循环, 继续下次循环



### if语句格式

```java
// 格式1
if (关系表达式) {
	语句体;   // 关系表达式为true时执行
}

// 格式2
if (关系表达式) {
	语句体1;   // 关系表达式为true时执行
} else {
	语句体2;   // 关系表达式为false时执行
}

// 格式3
if (关系表达式1) {
	语句体1;   // 关系表达式1为true时执行
} else if (关系表达式2) {
	语句体2;   // 关系表达式2为true时执行
}
...
} else {
	语句体n;   // 以上关系表达式都为false时执行
}
```

### switch语句的格式

```java
// 格式
switch (表达式) {
	case 值1:    // 当表达式的值为值1时执行
		语句体1;
		break;
	case 值2:    // 当表达式的值为值2时执行
		语句体2;
		break;
	...
	default:    // 当表达式的值不能匹配上面的case时执行
		语句体n;
		break;
} 
```


### for循环格式

```java
// 格式
for (初始化语句; 判断条件语句; 控制条件语句) {
	循环体语句;
}

// 示例
for (int i = 1; i <= 5; i++) {
	// 获取1~5的数值
}
```


### while循环格式

```java
// 普通格式
while (判断条件语句) {
	循环体语句;
}

// 扩展格式
初始化语句;
while (判断条件语句) {
	循环体语句;
	控制条件语句;
}

// 示例
int i = 1;
while (i <= 5) {
	syso(i);  // 打印1~5
	i++;
}
```


### do...while格式

```java
// 普通格式
do {
	循环体语句;
} while (判断条件语句);


// 扩展格式
初始化语句;
do {
	循环体语句;
	控制条件语句;
} while (判断条件语句);

// 示例
int i = 1;
do {
	syso(i);
	i++;
} while (i <= 5);
```

## 编程思维: 4种问题的解决方式

* 求和
* 判断符合条件
* 计数
* 交换2个变量的值


* 编程思维: 求和问题
	1. 定义一个变量sum, 用来存储求和的结果
	2. for循环获取不同的数
	3. 在for循环的循环体中使用sum变量累加每个数
	4. for循环结束后, sum就是总和

```java
// 定义一个变量sum, 用来存储求和的结果
int sum = 0;
// for循环获取不同的数
for (int i = 0; i < 100; i++) {
	// 使用sum变量累加每个数
	sum += i;
}
// for循环结束后, sum就是总和
System.out.println("总和是" + sum);
```

* 编程思维: 求某些符合条件的数字的总和的思路
	1. 定义一个变量sum, 用来存储求和的结果
	2. for循环获取不同的数
	3. 在for循环的循环体中使用if判断符合条件的数
	4. 对符合条件的数使用sum变量累加每个数
	5. for循环结束后sum就是符合条件的数的总和

```java
// 定义一个变量sum, 用来存储求和的结果
int sum = 0;
// for循环获取不同的数
for (int i = 0; i < 100; i++) {
	// 使用if判断符合条件的数
	if (条件) {
		// 使用sum变量累加每个数
		sum += i;
	}
}
// for循环结束后, sum就是总和
System.out.println("总和是" + sum);
```

* 编程思维: 统计个数
	1. 定义一个变量count, 用于统计个数
	2. for循环
	3. 在for循环中使用if判断出符合条件的数
	4. 当符合条件时, count++
	5. for循环结束后, count的值就是个数

```java
// 定义一个变量count, 用于统计个数
int count = 0;
// for循环
for (int i = 0; i < 100; i++) {
	// 使用if判断出符合条件的数
	if (条件) {
		// 当符合条件时, count++
		count++;
	}
}
// for循环结束后, count的值就是个数
System.out.println("个数是" + count);
```

* 编程思维: 交换2个变量的值
	1. 定义一个临时变量temp
	2. 先将a的值存入temp
	3. 再将b的值存入a
	4. 最后将temp的值存入b

```java
// 假设有2个变量a, b, 要交换他们的值
int a = 1;
int b = 2;

// 定义一个临时变量, 用于作为临时存储空间, 先保存a的值
int temp = a;
a = b;
b = temp;

// 交换完毕
```